#include "IZ_keyboard.h"

static IZ_ConfigItem keyboard_config_items[IZ_PLAYERS * IZ_CONTROLS + 1];

void IZ_KeyboardSerializeControl(i32 value, char control[128]) {
	const char* serialized = SDL_GetKeyName(value);
	memcpy_s(control, 128, serialized, 128);
}

i32 IZ_KeyboardDeserializeControl(const char* control) {
	return SDL_GetKeyFromName(control);
}

void IZ_KeyboardHandleKeyUpDownEvents(IZ_KeyboardState* state, IZ_Action* action, SDL_Event e) {
	u8 control_index;
	for (control_index = 0; control_index < IZ_CONTROLS; control_index += 1) {
		if (e.key.keysym.sym == state->config.control_mapping[control_index]) {
			const u16 bitflag = (0x1 << control_index);
			if (e.type == SDL_KEYDOWN) {
				*action |= bitflag;
				return;
			}
			if (e.type == SDL_KEYUP) {
				*action &= ~bitflag;
				return;
			}
		}
	}
}

void IZ_KeyboardHandleEvents(IZ_KeyboardState(* state)[IZ_PLAYERS], IZ_Action(* action)[IZ_PLAYERS], SDL_Event e) {
	for (u8 player_index = 0; player_index < IZ_PLAYERS; player_index += 1) {
		IZ_KeyboardHandleKeyUpDownEvents(&(*state)[player_index], &(*action)[player_index], e);
	}
}

void IZ_KeyboardBindStateToConfig(IZ_KeyboardState(* state)[IZ_PLAYERS], IZ_ConfigItem config_items[]) {
	u8 player_index;
	u8 control_index;
	for (player_index = 0; player_index < IZ_PLAYERS; player_index += 1) {
		for (control_index = 0; control_index < IZ_CONTROLS; control_index += 1) {
			config_items[player_index * IZ_CONTROLS + control_index].dest = &((*state)[player_index].config.control_mapping[control_index]);
		}
	}
}

IZ_ProcedureResult IZ_KeyboardSaveConfig(IZ_KeyboardState(* state)[IZ_PLAYERS], const char* config_path) {
	IZ_KeyboardBindStateToConfig(state, keyboard_config_items);
	return IZ_ConfigSave(keyboard_config_items, config_path);
}

void IZ_KeyboardInitializeConfigItems(IZ_ConfigItem config_items[]) {
	u8 player_index;
	u8 control_index;
	char* control_mapping_section_name;
	for (player_index = 0; player_index < IZ_PLAYERS; player_index += 1) {
		control_mapping_section_name = SDL_malloc(sizeof(char) * 64);
		sprintf_s(control_mapping_section_name, 64, "Keyboard.%d.ControlMapping", player_index);
		for (control_index = 0; control_index < IZ_CONTROLS; control_index += 1) {
			config_items[player_index * IZ_CONTROLS + control_index] = (IZ_ConfigItem) {
				IZ_CONFIG_TYPE_I32,
				sizeof(i32),
				control_mapping_section_name,
				IZ_ACTION_NAMES[control_index],
				NULL,
				&IZ_KEYBOARD_DEFAULT_STATE[player_index].config.control_mapping[control_index],
				NULL,
				{
					.serialize = IZ_KeyboardSerializeControl,
					.deserialize = IZ_KeyboardDeserializeControl,
				},
				NULL,
			};
		}
	}

	config_items[IZ_PLAYERS * IZ_CONTROLS] = IZ_CONFIG_ITEM_NULL;
}

IZ_ProcedureResult IZ_KeyboardInitializeConfig(IZ_KeyboardState(* state)[IZ_PLAYERS], const char* config_path, u8 argc, const char* argv[]) {
	IZ_KeyboardInitializeConfigItems(keyboard_config_items);
	IZ_KeyboardBindStateToConfig(state, keyboard_config_items);
	if (IZ_ConfigInitialize(keyboard_config_items, config_path, argc, argv) < 0) {
		return -1;
	}
	return 0;
}

IZ_ProcedureResult IZ_KeyboardInitialize(IZ_KeyboardState(* state)[IZ_PLAYERS], const char* config_path, u8 argc, const char* argv[]) {
	SDL_memcpy(state, &IZ_KEYBOARD_DEFAULT_STATE, sizeof(IZ_KeyboardState) * IZ_PLAYERS);
	if (IZ_KeyboardInitializeConfig(state, config_path, argc, argv) < 0) {
		return -2;
	}

	return 0;
}
