#ifndef IZ_VIDEO_H
#define IZ_VIDEO_H

#include <stdio.h>
#include "minIni.h"
#include "SDL_render.h"
#include <SDL_image.h>

#include "../../../net/IZ_net_client.h"
#include "../../../config/IZ_config.h"
#include "../../../common/IZ_common.h"
#include "../../input/IZ_input.h"

#define MAX_ACTIVE_SPRITES 512u

typedef enum {
	// eyecandy, e.g. sparks
	IZ_VIDEO_SPRITE_PRIORITY_LOWEST,
	// bottom backgrounds
	IZ_VIDEO_SPRITE_PRIORITY_LOWER,
	// top backgrounds
	IZ_VIDEO_SPRITE_PRIORITY_LOW,
	// projectiles
	IZ_VIDEO_SPRITE_PRIORITY_MEDIUM,
	// foreground objects
	IZ_VIDEO_SPRITE_PRIORITY_HIGH,
	// pickups
	IZ_VIDEO_SPRITE_PRIORITY_HIGHER,
	// player, enemies, weapons
	IZ_VIDEO_SPRITE_PRIORITY_HIGHEST,
} IZ_VideoSpritePriority;

// TODO properly define sprites
typedef struct {
	SDL_Texture* texture;
	f32 scale_factor;
	f32 rotate_degrees;
} IZ_Sprite;

typedef struct {
	u16 width;
	u16 height;
	u8 max_fps;
} IZ_VideoConfig;

typedef struct {
	void* user_data;
	IZ_VideoConfig config;
	u64 last_update_at;
	SDL_Window* window;
	SDL_Renderer* renderer;
	IZ_Sprite* active_sprites[MAX_ACTIVE_SPRITES];
} IZ_VideoState;

static const IZ_VideoState IZ_VIDEO_DEFAULT_STATE = {
	.user_data = NULL,
	.config = {
		.width = 320u,
		.height = 240u,
		.max_fps = 30u,
	},
	.last_update_at = 0,
	.renderer = NULL,
	.window = NULL,
	.active_sprites = {},
};

IZ_ProcedureResult IZ_VideoInitialize(IZ_VideoState*, void*, const char*, u8, const char*[]);

IZ_ProcedureResult IZ_VideoSaveConfig(IZ_VideoState*, const char*);

// TODO implement
void IZ_VideoLoadTexture(IZ_VideoState*, const char*, SDL_Texture**);

void IZ_VideoTeardown(IZ_VideoState*);

#endif
